using System;
using System.Collections.Generic;
using System.Text;

namespace LinearProgramming
{
    public class PenaltyFunction
    {
        LinearSpec ls;
        readonly Variable Var;
        readonly double[] Xs;
        readonly double[] Gs;

        List<Constraint> constraints = new List<Constraint>();
        List<ObjFunctionSummand> objFunctionSummands = new List<ObjFunctionSummand>();

        internal PenaltyFunction(LinearSpec ls, Variable var, double[] xs, double[] gs)
        {
            if (var.ls != ls)
                throw new Exception("The variable must belong to the same linear specification as the penalty function.");
            if (xs.Length + 1 != gs.Length)
                throw new Exception("The number of sampling points must be exactly one less than the number of gradients.");
            for (int i = 1; i < gs.Length; i++)
                if (gs[i - 1] > gs[i]) throw new Exception("Penalty function must be concave.");

            Xs = xs;
            Gs = gs;

            constraints.Add(ls.AddSoftConstraint(
                new double[] { 1 }, new Variable[] { var },
                OperatorType.EQ, xs[0], -gs[0], gs[1]));

            for (int i = 1; i < gs.Length - 1; i++)
            {
                Variable dpos = ls.AddVariable();
                constraints.Add(ls.AddConstraint(
                    new double[] { 1, -1 },
                    new Variable[] { var, dpos },
                    OperatorType.LE, xs[i]));
                objFunctionSummands.Add(ls.AddObjFunctionSummand(gs[i + 1] - gs[i], dpos));
            }
        }

        public void Remove()
        {
            foreach (Constraint c in constraints) c.Remove();
            foreach (ObjFunctionSummand s in objFunctionSummands) s.Remove();
        }
    }
}
